#!/bin/sh

#Copyright (c) 2006 SUNY at Stony Brook
#Copyright (c) 2006 Avishay Traeger, Nikolai Joukov, Josef Sipek, and Erez Zadok

ACCOUNTSCONF=$1
BACKUPSCONF=$2

source common

if [ -z $BACKUPSCONF ] ; then
	echo "Usage: mailbackup.sh accounts.conf backups.conf"
	exit 1
fi

if [ ! -f $ACCOUNTSCONF ] ; then
	echo "ERROR: accounts file does not exist"
	exit 1
fi

if [ ! -f $BACKUPSCONF ] ; then
	echo "ERROR: backups file does not exist"
	exit 1
fi

source $BACKUPSCONF
source $ACCOUNTSCONF

#Get MailBackup password
echo
echo -n "Please enter MailBackup password: "
stty -echo
read PASSWD
stty echo
echo

NUMBACKELEM=${#backup[@]}
NUMACCOUNTELEM=${#accounts[@]}
i=0
while [ "$i" -lt "$NUMBACKELEM" ]
do
	ACCOUNT=${backup[$i]}
	FILESTOBACK=${backup[$i+1]}
	BACKOPTS=${backup[$i+2]}
	TIMESTAMP=${backup[$i+3]}

	#Parse options
	j=1
	COMPRESS=0
	ENCRYPT=0
	OPT=`echo $BACKOPTS | cut -d "," -f $j`
	while [ ! -z $OPT ]
	do
		if [ $OPT = "comp" ] ; then
			COMPRESS=1
		elif [ $OPT = "enc" ] ; then
			ENCRYPT=1
		fi
		
		j=`expr $j + 1`
		OPT=`echo $BACKOPTS | cut -d "," -f $j`

		MULTOPS=`echo $BACKOPTS | grep -c ","`
		if [ "$MULTOPS" = "0" ] ; then
			OPT=""
		fi
	done

	#Find account in the conf file
	j=0
	FOUNDACCOUNT=0
	while [ "$j" -lt "$NUMACCOUNTELEM" ]
	do
		if [ "${accounts[$j]}" = "$ACCOUNT" ] ; then
			FOUNDACCOUNT=1
			ACCOUNTTYPE=${accounts[$j+1]}
			ACCOUNTOPT=${accounts[$j+2]}
			break
		fi
		j=`expr $j + $ELEMINACCCONF`
	done

	if [ "$FOUNDACCOUNT" = "0" ] ; then
		echo "ERROR: Failed to find account information for $ACCOUNT"
		rm -f $FILETOSEND $FILESFILE
		exit 1
	fi

	#Create tar file
	DATE=`date +%F`
	TIME=`date +%T`
	DATESTR=$DATE
	TIMESTR=`date -d "$DATE $TIME" +%k-%M`
	if [ "$COMPRESS" = "1" ] ; then
		TARFILE=$DATESTR--$TIMESTR.tar.gz
		TAROPTS="cvzf"
	else
		TARFILE=$DATESTR--$TIMESTR.tar
		TAROPTS="cvf"
	fi
	FILETOSEND=$TARFILE

	if [ -z "$TIMESTAMP" ] ; then
		TARMTIME=""
	else
		TARMTIME="--newer-mtime $TIMESTAMP"
	fi

	FILELIST=`echo $FILESTOBACK | sed s/';'/' '/g`
	FILESFILE=$TARFILE.files
	tar $TAROPTS $TARFILE $TARMTIME $FILELIST > $FILESFILE 2> /dev/null

	#Encrypt if necessary
	if [ "$ENCRYPT" = "1" ] ; then
		ENCFILE=$TARFILE.enc
		echo "$PASSWD" | gpg -a --batch -o $ENCFILE -c --passphrase-fd 0 $TARFILE 2>&1
		rm -f $TARFILE
		FILETOSEND=$ENCFILE
	else
		ENCFILE=$TARFILE.cmp
		uuencode $TARFILE < $TARFILE > $ENCFILE
		rm -f $TARFILE
		FILETOSEND=$ENCFILE
	fi

	#Append checksum to files file
	echo "" >> $FILESFILE
	echo "[SHA1SUM]" >> $FILESFILE
	sha1sum -b $FILETOSEND >> $FILESFILE

	# append End of List marker & flags
	echo -n "EOL:" >> $FILESFILE
	if [ "$ENCRYPT" = "1" ] ; then
		echo -n "ENCRYPTED:" >> $FILESFILE
	fi
	if [ "$COMPRESS" = "1" ] ; then
		echo -n "COMPRESSED:" >> $FILESFILE
	fi
	echo "" >> $FILESFILE
	
	if [ "$ACCOUNTTYPE" = "gmailfs" ] ; then
		if [ ! -d $ACCOUNTOPT ] ; then
			echo "ERROR: $ACCOUNTOPT is not a directory"
			rm -f $FILETOSEND $FILESFILE
			exit 1
		fi
		mv $FILETOSEND $FILESFILE $ACCOUNTOPT
	elif [ "$ACCOUNTTYPE" = "smtp" ] ; then
		cat $FILESFILE $FILETOSEND > $FILETOSEND.tmp
		mv $FILETOSEND{.tmp,}
		./blobify.py $FILETOSEND $FILETOSEND.part $ACCOUNTOPT
		for x in $FILETOSEND.part*
		do
			subj=`cat $x | head -1`
			next=`cat $x | head -2 | tail -1`
			(cat $x; echo "") | mail -s "$subj" $ACCOUNT

			echo "Sending $subj -> $next"
		done

		# now, send the index file
		hash=`cat $FILETOSEND.part.000|head -1`
		echo $hash | mail -s "$FILETOSEND" $ACCOUNT
		echo "Sending $FILETOSEND -> $hash"
	else
		echo "ERROR: Unknown account type!"
	fi

	#Update timestamp in backups file
	TIMESTAMP="$DATE $TIME"
	TINDEX=`expr $i + 3`
	sed s/^backup'\['$TINDEX'\]'=[-0-9:' '\"]*/backup[$TINDEX]=\"$DATE' '$TIME\"/g $BACKUPSCONF > $BACKUPSCONF-$$.tmp
	mv $BACKUPSCONF-$$.tmp $BACKUPSCONF

	i=`expr $i + $ELEMINBACKCONF`
done
