#!/usr/bin/env python

#Copyright (c) 2006 SUNY at Stony Brook
#Copyright (c) 2006 Avishay Traeger, Nikolai Joukov, Josef Sipek, and Erez Zadok

import sys, os
import sha

def err(s):
	sys.stderr.write("%s\n" % (s,))

if len(sys.argv) != 3:
	err("%s mbox /file/to/restore" % (sys.argv[0],))
	sys.exit(1)

(mbox, restorefile) = sys.argv[1:]

def lookslikehash(s):
	return len(s.strip("0123456789ABCDEFabcdef")) == 0

def addemail(i, e, h, b):
	e[h["subject"]] = (h, b[:-1])
	if not lookslikehash(h["subject"]):
		i.append(h["subject"])

fm = open(mbox, "rb")

START	= 0
HEADERS	= 1
BODY	= 2

index = []
emails = {}
headers = {}
state = START
for l in fm.readlines():
	cl = l[:-1]
	if l.startswith("From "):
		if state != START:
			addemail(index, emails, headers, body)
		state = HEADERS
		headers = {}
		body = ""
		lines = 0
	elif l.startswith("\t") and state == HEADERS:
		pass # more headers
	elif cl == "" and state == HEADERS:
		state = BODY
	elif state == HEADERS:
		if l.startswith("Lines: "):
			headers["lines"] = int(l[7:])
		elif l.startswith("Subject: "):
			headers["subject"] = l[9:-1]
	else:
		body += l

if state == BODY:
	addemail(index, emails, headers, body)

fm.close()

# alright at this point, we have the entire contents of the mbox in memory,
# let's do something useful with it

def have_list_end(s):
	# FIXME: make this actually check of the end of list of files
	return False

for i in index:
	# let's assemble the file list
	hash = emails[i][1].strip("\n")

	text = ""
	while not have_list_end(text):
		(h, n, l, t) = emails[hash][1].split("\n", 3)

		if hash != h or len(t) != int(l):
			err("Something is terribly wrong! (%s != %s or %d != %d)" % (hash, h, len(t), int(l)))
			sys.exit(1)

		actualhash = sha.sha(t).hexdigest()
		if actualhash != h:
			err("Found corrupt blob! SHA1=%s, should be %s" % (actualhash, h))
			sys.exit(1)

		text += t
		if n == ("0" * 40):
			# end of stream!
			break

		err("Chain continues at %s" % (n,))
		hash = n
	
	err("Done. got %d chars from %s" % (len(text),i))

	filelist = text[:text.find("EOL:")].split("\n")

	# kill empty lines
	while True:
		try:
			filelist.remove("")
		except ValueError:
			break
	# get & kill hash
	hash = filelist[-1].split(" ")[0]
	filelist = filelist[:-2]

	data = text[text.find("\n", text.find("EOL:"))+1:]
	flags = text[text.find("EOL:"): text.find("\n", text.find("EOL:"))]

	internal = sha.sha(data).hexdigest()
	if hash != internal:
		err("Internal hash doesn't match data (%s != %s)" % (hash, internal))
		sys.exit(1)

	found = False
	for f in filelist:
		if restorefile == os.path.join("/", f):
			err("Looks like we got a winner!")
			err("restorefile = %s" % (restorefile,))
			err("f = %s" % (f,))
			found = True
			break
	
	if not found:
		# ok, this file doesn't have it...let's try next one
		continue
	
	# save the file
	f = open("%s.fetched" % (i,), "wb")
	f.write(data)
	f.close()

	sys.stdout.write("%s %s.fetched\n" % (flags,i))

	sys.exit(0)

err("Couldn't find the file anywhere")
sys.exit(1)

